-- ---------------------------------------------------
-- Tidy Plates: Grey - Theme Definition
-- ---------------------------------------------------

-- Start Theme
SetCVar("threatWarning", 3)
SetCVar("ShowClassColorInNameplate", 1)
SetCVar("CameraDistanceMaxFactor", 6)

local path = "Interface\\Addons\\TidyPlates_Grey"
local NO_MODE, TANK_MODE, DPS_MODE, PVP_MODE = 1, 2, 3, 4
local HEALTH_PCT, HEALTH_TOTAL, HEALTH_DEF = 2, 3, 4
local WIDGET_NONE, WIDGET_COMBO, WIDGET_THREAT = 1, 2, 3

if not TidyPlatesGreyVariables then TidyPlatesGreyVariables = {} end
TidyPlatesGreyVariables = {Mode = 1, HealthText = 1, AutoHide = false, WidgetFunction = WIDGET_NONE, GlobalScale = 1}

------------------------------
-- Backup the Default Variables
------------------------------

local function copytable(original)
	local duplicate = {}
	for key, value in pairs(original) do
		if type(value) == "table" then duplicate[key] = copytable(value)
		else duplicate[key] = value end
	end
	return duplicate
end

local TidyPlatesGreyVariablesDefaults =copytable( TidyPlatesGreyVariables )

--[[ 
To Do: 
	* Iconlist
		- Acidmaw/Dreadscale, etc.
		- "Watch your step"
		- Healer, Melee, Ranged
		- Faction Champs
	* Threat:
		- No threat for non-elites in tank mode
		- No threat for friendlies
--]]

------------------------------
-- Combo Point Widget
------------------------------
--  [[
local combopath = path.."\\Art\\ComboPoints\\"
local COMBO_ART = {
"Blue",
"Green",
"Yellow",
"Orange",
"Red",
}

local function UpdateComboPointFrame(frame, points)
		--local points = 0
		--if UnitExists("target") then points = GetComboPoints("player", "target") end
		--print("Here")
		if points > 0 then 
			frame.Icon:SetTexture(combopath..COMBO_ART[points]) 
			frame:Show()
		else frame:Hide() end	
end

local function CreateComboPointFrame(parent)
	local frame = CreateFrame("Frame", nil, parent)
	frame:SetHeight(64)
	frame:SetWidth(64)
	frame:RegisterEvent("PLAYER_COMBO_POINTS")
	frame:SetScript("OnEvent", function() TidyPlates:ForceUpdateFrame(parent.secureframe) end)
	frame.Icon = frame:CreateTexture(nil, "OVERLAY")
	frame.Icon:SetAllPoints(frame)
	frame:Hide()
	frame.UpdateCombo = UpdateComboPointFrame
	return frame
end
--]]

---------------
-- Threat Functions
---------------
--GameTooltip:HookScript("OnTooltipSetUnit", function(self) TidyPlates:Update() end)
local threatpath = "Interface\\Addons\\TidyPlates_Grey\\Art\\RingThreat\\"
local threatfont = "Interface\\Addons\\TidyPlates_Grey\\Art\\LiberationSans-Regular.ttf"
local TANK_SEGMENT_ART = {
HIGHTANK = threatpath.."Red",
MEDIUMTANK = threatpath.."Orange",
LOWTANK = threatpath.."Yellow",
HIGHDPS = threatpath.."Green",
MEDIUMDPS = threatpath.."Blue",
LOWDPS = threatpath.."Blue",
UNKNOWN = threatpath.."Unknown",
}

local function SetFade(frame)
	if GetTime() > frame.FadeTime then
		frame:Hide()
		frame:SetScript("OnUpdate", nil)
	end
end

local function GetThreatSegment(threat)
	if not threat then return nil end
	if threat > 135 then return "HIGHTANK"
	elseif threat > 110 then return "MEDIUMTANK"
	elseif threat > 100 then return "LOWTANK"
	elseif threat > 85 then return "HIGHDPS"
	elseif threat > 55 then return "MEDIUMDPS"
	else return "LOWDPS"  end
end

local function GetRelativeThreat(unit)
	if not UnitExists(unit) then return end
	local _, group, size, index, unitid
	local playerthreat, leaderthreat, tempthreat, leadername = 0,0,0, nil
	_, _,  playerthreat, _, _ = UnitDetailedThreatSituation("player", unit)
	if HasPetUI() then _, _,  leaderthreat, _, _ = UnitDetailedThreatSituation("pet", unit); leadername = "pet" end
	if UnitInRaid("player") then group = "raid"; size = GetNumRaidMembers() - 1
	elseif UnitInParty("player") then group = "party"; size = GetNumPartyMembers()
	else group = nil end
	if group then
		for index = 1, size do
			unitid = group..index
			_, _, tempthreat , _, _ = UnitDetailedThreatSituation(unitid, unit)
			if tempthreat and tempthreat > leaderthreat then 
				leaderthreat = tempthreat 
				leadername = unitid
				end
		end
	end
	if playerthreat and leaderthreat then
		if playerthreat == 100 then return playerthreat + (100-leaderthreat), nil
		else return playerthreat, leadername end
	else return end
end


---------------
-- Threat Widget
---------------
--  [[
local function UpdateThreatWidget(frame, unitid)
	if unitid then
		local threat, topholder = GetRelativeThreat(unitid) 
		local threatsegment = GetThreatSegment(threat)
		if threatsegment then
			-- Set Indicator
			frame:Show()
			frame.ThreatIcon:SetTexture(TANK_SEGMENT_ART[threatsegment])
			frame.ThreatText:SetText(floor(threat))
			if topholder then 
				frame.TargetText:SetText(UnitName(topholder))
				--local _, class = UnitClass(topholder)
				--local classcolor = RAID_CLASS_COLORS[class]
				--frame.TargetText:SetTextColor(classcolor.r, classcolor.g, classcolor.b) 
			else frame.TargetText:SetText("") end
			-- Set Fading
			frame.FadeTime = GetTime() + 3
			frame:SetScript("OnUpdate", SetFade)
		else frame:Hide() end
	elseif (GetTime() > frame.FadeTime) or (not InCombatLockdown()) then frame:Hide() end
end

local function CreateThreatWidget(parent, showname, hangtime, art)
	local frame = CreateFrame("Frame", nil, parent)
	frame:SetWidth(64)
	frame:SetHeight(64)
	-- Icon
	frame.ThreatIcon = frame:CreateTexture(nil, "OVERLAY")
	frame.ThreatIcon:SetAllPoints(frame)
	-- Threat Text
	frame.ThreatText = frame:CreateFontString(nil, "OVERLAY")
	frame.ThreatText:SetFont(threatfont, 8, "NONE")
	frame.ThreatText:SetPoint("CENTER",frame.ThreatIcon,"CENTER", -1, 0)
	frame.ThreatText:SetWidth(20)
	frame.ThreatText:SetHeight(20)
	-- Target Text
	frame.TargetText = frame:CreateFontString(nil, "OVERLAY")
	frame.TargetText:SetFont(threatfont, 8, "NONE")
	frame.TargetText:SetShadowOffset(1, -1)
	frame.TargetText:SetShadowColor(0,0,0,1)
	frame.TargetText:SetPoint("BOTTOM",frame,"TOP", 1, -25)
	frame.TargetText:SetWidth(50)
	frame.TargetText:SetHeight(20)
	-- Setup
	frame.FadeTime = 0
	frame:Hide()
	--parent.bars.health:RegisterEvent("CURSOR_UPDATE")
	--frame:SetScript("OnEvent", function() TidyPlates:ForceUpdateFrame(parent.secureframe)  end)
	frame.UpdateThreat = UpdateThreatWidget
	return frame
end
--]]

---------------
-- Mana Widget
---------------
--[[
local combopath = "Interface\\Addons\\TidyPlates_Grey\\Art\\ComboPoints\\"
local COMBO_ART = {
"Blue",
"Green",
"Yellow",
"Orange",
"Red",
}

local function UpdateManaWidget(self, points)
		if points > 0 then 
			self.Icon:SetTexture(combopath..COMBO_ART[points]) 
			self:Show()
		else self:Hide() end	
end

local function CreateManaWidget(parent)
	local frame = CreateFrame("Frame", nil, parent)
	frame:RegisterEvent("CURSOR_UPDATE")
	frame:RegisterEvent("PLAYER_COMBO_POINTS")
	frame:SetScript("OnEvent", function() TidyPlates:ForceUpdateFrame(parent.secureframe)  end)
	frame.Icon = frame:CreateTexture(nil, "OVERLAY")
	frame.Icon:SetPoint("CENTER",parent,"TOP")
	frame.Icon:SetWidth(64)
	frame.Icon:SetHeight(64)
	frame:Hide()
	frame.UpdateMana = UpdateManaWidget
	return frame
end
--]]

---------------
-- Text Helpers
---------------

local function valueToString(value)
	if value >= 1000000 then return format('%.1fm', value / 1000000)
	elseif value >= 1000 then return format('%.1fk', value / 1000)
	else return value end
end

---------------
-- Text Delegates
---------------

local function SpellTextDelegate(unit)
	local spellname
	if unit.isCasting then 
		spellname = UnitCastingInfo("target") or UnitChannelInfo("target")
		return spellname
	else return "" end
end

local function HealthTextDelegate(unit)
	if TidyPlatesGreyVariables.HealthText == HEALTH_PCT then
		if unit.health ~= unit.healthmax then 
			return "%"..ceil(100*(unit.health/unit.healthmax))
		else return nil end
	elseif TidyPlatesGreyVariables.HealthText == HEALTH_TOTAL then
		return valueToString(unit.health)
	elseif TidyPlatesGreyVariables.HealthText == HEALTH_DEF then
		if unit.health ~= unit.healthmax then return "-"..valueToString(unit.healthmax - unit.health) end
	else return "" end
end

---------------
-- Graphics Delegates
---------------
local classiconpath = path.."\\Art\\Class\\"
local function ArtDelegate(unit)
	if TidyPlatesGreyVariables.Mode == 4 then
		if unit.class and (unit.class ~= "UNKNOWN") then 
			return classiconpath..unit.class  
		end
	else return nil end
end		

local function ScaleDelegate(unit)
	local basescale = TidyPlatesGreyVariables.GlobalScale
	if unit.isCasting then return basescale end
	if InCombatLockdown() then
		-- Set Scale for Tank Mode
		if TidyPlatesGreyVariables.Mode == TANK_MODE and unit.reaction == "HOSTILE" and InCombatLockdown() then
			if (not unit.isElite) then basescale = basescale * .8 end
			if unit.threatSituation == "LOW" and unit.isInCombat then return basescale * 1.2
			elseif unit.threatSituation == "HIGH" then return basescale * 0.8 end
		-- Set Scale for DPS Mode
		elseif TidyPlatesGreyVariables.Mode == DPS_MODE then
			if (not unit.isElite) then basescale = basescale * .8 end
			if unit.threatSituation == "HIGH" then return basescale * 1.2
			elseif unit.threatSituation == "LOW" then return basescale end
		elseif TidyPlatesGreyVariables.Mode == PVP_MODE then end
	end
	return basescale	
end

local function AlphaDelegate(unit)
	local basealpha = 1
	if unit.isCasting or unit.isMarked then return 1 end
	-- Blah blah blah
	if InCombatLockdown() then
		-- Mode-Specific Filters
		if TidyPlatesGreyVariables.Mode == TANK_MODE then
			if (not unit.isElite) then basealpha = .8 end
			if unit.threatSituation == "HIGH" then return basealpha * 0.9 end
		elseif TidyPlatesGreyVariables.Mode == DPS_MODE then
			if (not unit.isElite) then basealpha = .8 end
		end
	end
	if unit.reaction == "NEUTRAL" and unit.threatSituation == "LOW" then return 0 
	elseif (not unit.isInCombat) and InCombatLockdown() then return basealpha * .5
	else return basealpha end
end

---------------
-- Widgets
---------------
local function OnInitializeDelegate(plate)
	if TidyPlatesGreyVariables.WidgetFunction == WIDGET_COMBO then
		plate.ComboPointFrame = CreateComboPointFrame(plate)
		plate.ComboPointFrame:SetPoint("CENTER",plate, "TOP", 35, 4)
	elseif TidyPlatesGreyVariables.WidgetFunction == WIDGET_THREAT then
		plate.ThreatFrame = CreateThreatWidget(plate)
		plate.ThreatFrame:SetPoint("CENTER",plate,"TOP", 25, 3)
		plate.ThreatFrame:SetScale(1.2)
	end
 end

local function OnUpdateDelegate(plate, unit)
	if TidyPlatesGreyVariables.WidgetFunction == WIDGET_COMBO then
		if not plate.ComboPointFrame then OnInitializeDelegate(plate) end
		local combos = 0
		if unit.alpha == 1 and UnitExists("target") then combos = GetComboPoints("player", "target") end
		plate.ComboPointFrame:UpdateCombo(combos)
	elseif TidyPlatesGreyVariables.WidgetFunction == WIDGET_THREAT then
		if TidyPlatesGreyVariables.Mode == TANK_MODE and ((not unit.isElite) or unit.reaction == "FRIENDLY") then return end
		if not plate.ThreatFrame then OnInitializeDelegate(plate) end
		local unitid = nil
		if unit.isMouseover then plate.ThreatFrame:UpdateThreat("mouseover")
		elseif unit.alpha == 1 and UnitExists("target") then plate.ThreatFrame:UpdateThreat("target")
		else plate.ThreatFrame:UpdateThreat(nil) end
	end
end

---------------
-- Theme definition (Just uses the default art)
---------------

local themedata = {
	hitbox = {
		width = 140,
		height = 35,
	},
	options = {
		showName = true,
		showLevel = true,
		showSpecialText = true,
		showSpecialText2 = true,
		showSpecialArt = true,
	},
	SetSpecialText = HealthTextDelegate,
	SetSpecialText2 = SpellTextDelegate,
	SetSpecialArt = ArtDelegate,
	SetScale = ScaleDelegate,
	SetAlpha = AlphaDelegate,
	OnUpdate = OnUpdateDelegate,
	OnInitialize = OnInitializeDelegate,
}

TidyPlatesThemeList["Grey"] = themedata

------------------------------------------------------------------
-- Panel
------------------------------------------------------------------

-- Main Panel
local panel = PanelHelpers:CreatePanelFrame( "TidyPlatesGreyInterfaceOptions", "Grey Theme" )
panel.parent = "Tidy Plates"

---- MODE 
panel.radiobuttonlabel = panel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
panel.radiobuttonlabel:SetPoint("TOPLEFT", panel, "TOPLEFT", 35, -65)
panel.radiobuttonlabel:SetText("Display Mode:")
local radiolist = {"Default: |cFFFFFFFF No Scaling or extra art.",
		"Tank: |cFFFFFFFF Reduces scale and opacity of units that are attacking you. ",
		"Damage: |cFFFFFFFF Increases scale and opacity of units that are attacking you.", 
		"PVP: |cFFFFFFFF  Shows enemy class icon on the nameplate. "}
panel.radiobuttons = PanelHelpers:CreateRadioButtons("TidyPlatesGrey_ModeRadio", panel, 4, 1, 20, radiolist)
panel.radiobuttons[1]:SetPoint("TOPLEFT", panel.radiobuttonlabel, 0, -17)

---- WIDGET
panel.dropdownmenulabel = panel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
panel.dropdownmenulabel:SetPoint("TOPLEFT", panel, "TOPLEFT", 35, -225)
panel.dropdownmenulabel:SetText("Widget Function:")
local dropmenu = { 
				{ text = "None", notCheckable = 1 },
				{ text = "Combo Points", notCheckable = 1 } ,
				{ text = "Mouseover Threat", notCheckable = 1 } ,
			}
panel.dropdown = PanelHelpers:CreateDropdownFrame("TidyPlatesGrey_HealthTextDropdown", panel, dropmenu, 1)
panel.dropdown:SetPoint("TOPLEFT", panel.dropdownmenulabel, 0, -17)
panel.dropdown:SetValue(2)

--  SCALE
panel.Scalelabel = panel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
panel.Scalelabel:SetPoint("TOPLEFT", panel.dropdownmenulabel, "BOTTOMLEFT", 0, -50)
panel.Scalelabel:SetText("Nameplate Scale:")
panel.Scale = PanelHelpers:CreateSliderFrame("TidyPlatesGrey_ScaleSlider", panel)
panel.Scale:SetPoint("TOPLEFT", panel.Scalelabel, "BOTTOMLEFT", 5, -7)
panel.Scale:SetMinMaxValues(.5, 1.5)
panel.Scale:SetValue(1)
panel.Scale:SetValueStep(.1) 
panel.Scale.Low:SetText("50%")
panel.Scale.High:SetText("150%")
--]]

-- APPLY BUTTON
panel.apply = CreateFrame("Button", "TidyPlatesGrey_ApplyButton", panel, "UIPanelButtonTemplate2")
--panel.apply:SetPoint("TOPLEFT", panel.Scale, "BOTTOMLEFT", 0, -50)
panel.apply:SetPoint("BOTTOMRIGHT", panel, -15, 15)
panel.apply:SetText("Apply")
panel.apply:SetWidth(80)

-- Functions
panel.okay = function () 
	TidyPlatesGreyVariables.Mode = panel.radiobuttons:GetChecked()
	TidyPlatesGreyVariables.WidgetFunction = panel.dropdown.Value
	TidyPlatesGreyVariables.GlobalScale = panel.Scale:GetValue()
	TidyPlates:ForceUpdate()
end
-- APPLY BUTTON Script
panel.apply:SetScript("OnClick",panel.okay)

panel.refresh = function ()
	panel.radiobuttons:SetChecked(TidyPlatesGreyVariables.Mode)
	panel.dropdown:SetValue(TidyPlatesGreyVariables.WidgetFunction)
	panel.Scale:SetValue(TidyPlatesGreyVariables.GlobalScale)
end

panel:SetScript("OnEvent", function(self, event, ...) InterfaceOptions_AddCategory(panel) end)
panel:RegisterEvent("PLAYER_LOGIN")

--------------------------------------------  Slash Commands  --------------------------------------------
TidyPlatesGreySlashCommands = {}
TidyPlatesGreySlashCommands.tank = function() print("Tidy Plates Grey: Tank Mode Activated"); TidyPlatesGreyVariables.Mode = TANK_MODE end
TidyPlatesGreySlashCommands.dps = function() print("Tidy Plates Grey: DPS Mode Activated"); TidyPlatesGreyVariables.Mode = DPS_MODE end
TidyPlatesGreySlashCommands.pvp = function() print("Tidy Plates Grey: PVP Mode Activated"); TidyPlatesGreyVariables.Mode = PVP_MODE end
TidyPlatesGreySlashCommands.reset = function() print("Tidy Plates Grey: Variables Reset"); TidyPlatesGreyVariables = copytable( TidyPlatesGreyVariablesDefaults ) end
--------------------------------------------  Slash Commands  --------------------------------------------
function slash_TidyGrey(arg)
	if type(TidyPlatesGreySlashCommands[arg]) == 'function' then 
		TidyPlatesGreySlashCommands[arg]() 
		TidyPlates:ForceUpdate()
	else InterfaceOptionsFrame_OpenToCategory(panel);ChatFrame1:AddMessage('Tidy Plates Grey Commands;  /tidygrey {tank, dps, pvp}') end
end

SLASH_TIDYGREY1 = '/tidyGrey'
SlashCmdList['TIDYGREY'] = slash_TidyGrey;










